@@ -15,7 +15,7 @@ def treemap(node, fun, **kwargs):
     fun :: NodeData -> NodeData
     """
     node.data = fun(node.data, **kwargs)
-    node.kids = [treemap(k, fun, **kwargs) for k in node.kids]
+    node.kids = [treemap(k, fun, **kwargs) for k in node.kids if k is not None]
     return node
 
 
@@ -24,7 +24,7 @@ def treefold(node, fun, init, **kwargs):
     fun :: a -> NodeData -> a
     """
     x = fun(init, node.data, **kwargs)
-    for kid in node.kids:
+    for kid in [k for k in node.kids if k is not None]:
         x = treefold(kid, fun, x, **kwargs)
     return x
 
@@ -34,7 +34,7 @@ def treecut(node, fun, **kwargs):
     fun :: Node -> Node
     """
     node.kids = fun(node, **kwargs)
-    node.kids = [treecut(kid, fun, **kwargs) for kid in node.kids]
+    node.kids = [treecut(k, fun, **kwargs) for k in node.kids if k is not None]
     return node
 
 
@@ -49,7 +49,7 @@ def treepull(node, fun, **kwargs):
     if node.data.isLeaf:
         node.data = fun(node.data, [])
     else:
-        node.kids = [treepull(kid, fun, **kwargs) for kid in node.kids]
+        node.kids = [treepull(kid, fun, **kwargs) for kid in node.kids if kid is not None]
         node.data = fun(node.data, [kid.data for kid in node.kids], **kwargs)
     return node
 
@@ -68,6 +68,12 @@ def treepush(node, fun, **kwargs):
 
     return node
 
+def tips(node):
+    def _fun(b, x):
+        if x.isLeaf:
+            b.append(x.label)
+        return b
+    return treefold(node, _fun, [])
 
 def partition(xs, f):
     a = []
@@ -87,7 +93,9 @@ def clean(node, isRoot=True):
 
     def _clean(node, isRoot):
         # remove empty children
-        node.kids = [kid for kid in node.kids if kid.data.nleafs > 0]
+        node.kids = [
+            kid for kid in node.kids if kid is not None and kid.data.nleafs > 0
+        ]
         # remove all single-child nodes
         while len(node.kids) == 1:
             if node.data.length is not None and node.kids[0].data.length is not None:
@@ -138,23 +146,21 @@ def factorByField(node, field, default=None, sep="|"):
     return factorByLabel(node, _fun)
 
 
+def factorByCaptureFun(name, pat, default=None):
+    if name:
+        m = re.search(pat, name)
+        if m:
+            if m.groups():
+                # take the deepest match
+                return [x for x in list(m.groups()) if x is not None][-1]
+            else:
+                return m.groups(0)
+    return default
+
+
 def factorByCapture(node, pat, default=None):
     pat = re.compile(pat)
-
-    def _fun(name):
-        if name:
-            m = re.search(pat, name)
-            if m:
-                if m.groups(1):
-                    if isinstance(m.groups(1), str):
-                        return m.groups(1)
-                    else:
-                        return m.groups(1)[0]
-                else:
-                    return m.groups(0)
-        return default
-
-    return factorByLabel(node, _fun)
+    return factorByLabel(node, lambda x: factorByCaptureFun(x, pat, default=default))
 
 
 def factorByTable(node, filename, default=None):
@@ -358,10 +364,12 @@ def distribute(count, groups, sizes=None):
 
 
 def setNLeafs(node):
+    if not node:
+      return node
     if node.data.isLeaf:
         node.data.nleafs = 1
     else:
-        node.kids = [setNLeafs(kid) for kid in node.kids]
+        node.kids = [setNLeafs(kid) for kid in node.kids if kid is not None]
         node.data.nleafs = sum(kid.data.nleafs for kid in node.kids)
     return node
 
@@ -405,7 +413,7 @@ def sampleContext(node, keep=[], maxTips=5):
 
 
 def sampleParaphyletic(
-    node, proportion=None, scale=None, keep=[], keep_regex="", minTips=3, seed=None
+    node, proportion=None, scale=None, number=None, keep=[], keep_regex="", minTips=3, seed=None
 ):
     rng = random.Random(seed)
 
@@ -414,11 +422,17 @@ def sampleParaphyletic(
         def _sample(labels):
             return min(len(labels), max(minTips, math.ceil(proportion * len(labels))))
 
-    else:
+    elif scale:
 
         def _sample(labels):
             return min(len(labels), max(minTips, math.ceil(len(labels) ** scale)))
 
+    else:
+
+        def _sample(labels):
+            return min(len(labels), number)
+            
+
     def getLabels(node):
         def _collect(b, d):
             if d.isLeaf:
@@ -514,14 +528,16 @@ def sampleParaphyletic(
 
 
 def sampleProportional(
-    node, proportion=None, scale=None, keep=[], keep_regex="", minTips=3, seed=None
+    node, proportion=None, scale=None, number=None, keep=[], keep_regex="", minTips=3, seed=None
 ):
     rng = random.Random(seed)
 
     if proportion:
         count_fun = lambda xs: max(minTips, math.floor(len(xs) * proportion))
-    else:
+    elif scale:
         count_fun = lambda xs: max(minTips, math.floor(len(xs) ** scale))
+    else:
+        count_fun = lambda xs: min(len(xs), number)
 
     if keep_regex:
         keep_fun = lambda label: re.search(keep_regex, label)
@@ -533,54 +549,71 @@ def sampleProportional(
 
     # recursive sampler
     def _sampleProportional(node):
-        newkids = []
-        for kid in node.kids:
-            nfactors = len(kid.data.factorCount)
-            if nfactors == 0:
-                newkids.append(_sample(kid))
-            elif nfactors == 1:
-                if list(kid.data.factorCount.keys())[0] in keep:
-                    newkids.append(kid)
-                else:
-                    newkids.append(_sample(kid))
-            else:
-                newkids.append(_sampleProportional(kid))
-        node.kids = newkids
+        nfactors = len(node.data.factorCount)
+        if(nfactors == 0):
+          return _sample(node)
+        elif nfactors == 1:
+          if list(node.data.factorCount.keys())[0] in keep:
+            return node
+          else:
+            node = _sample(node)
+        else:
+            node.kids = [_sampleProportional(kid) for kid in node.kids]
         return node
 
     node = setFactorCounts(node)
     return clean(_sampleProportional(node))
 
-def colorTree(node, color):
-  def fun_(d):
-    d.form["!color"] = color
-    return d
 
-  return treemap(node, fun_)
+def colorTree(node, color):
+    def fun_(d):
+        d.form["!color"] = color
+        return d
+
+    return treemap(node, fun_)
+
 
 def colorMono(node, colormap):
     if len(node.data.factorCount) == 1:
-      label = list(node.data.factorCount.keys())[0] 
-      if label in colormap:
-        node = colorTree(node, colormap[label])
+        label = list(node.data.factorCount.keys())[0]
+        if label in colormap:
+            node = colorTree(node, colormap[label])
     else:
-      node.kids = [colorMono(kid, colormap) for kid in node.kids]
+        node.kids = [colorMono(kid, colormap) for kid in node.kids]
     return node
 
+
+def filterMono(node, condition, action):
+    if len(node.data.factorCount) == 1:
+      if condition(node):
+        node = action(node)
+    else:
+        node.kids = [filterMono(kid, condition, action) for kid in node.kids]
+    return node
+
+
 def intersectionOfSets(xss):
-  x = set(xss[0])
-  for y in xss[1:]:
-    x = x.intersection(set(y))
-  return x
+    try:
+        x = set(xss[0])
+        for y in xss[1:]:
+            x = x.intersection(set(y))
+    except:
+        x = set()
+    return x
+
 
 def colorPara(node, colormap):
     if len(node.data.factorCount) == 1:
-      label = list(node.data.factorCount.keys())[0] 
-      if label in colormap:
-        node = colorTree(node, colormap[label])
+        label = list(node.data.factorCount.keys())[0]
+        if label in colormap:
+            node = colorTree(node, colormap[label])
     else:
-      common = intersectionOfSets([k.data.factorCount.keys() for k in node.kids])
-      if len(common) == 1:
-        node = colorTree(node, colormap[list(common)[0]])
-      node.kids = [colorPara(kid, colormap) for kid in node.kids]
+        common = intersectionOfSets([k.data.factorCount.keys() for k in node.kids])
+        if len(common) == 1:
+            try:
+                color = colormap[list(common)[0]]
+                node = colorTree(node, colormap[list(common)[0]])
+            except KeyError:
+                pass
+        node.kids = [colorPara(kid, colormap) for kid in node.kids]
     return node
