@@ -81,6 +81,8 @@ def factorTree(
     factor_by_field=None,
     factor_by_table=None,
     default=None,
+    impute=False,
+    patristic=False,
 ):
     import smot.algorithm as alg
 
@@ -97,6 +99,12 @@ def factorTree(
     elif factor_by_table is not None:
         node = alg.factorByTable(node, filename=factor_by_table, default=default)
     node = alg.setFactorCounts(node)
+
+    if patristic:
+        node = alg.imputePatristicFactors(node)
+    elif impute:
+        node = alg.imputeFactors(node)
+
     return node
 
 
@@ -113,20 +121,19 @@ dec_tree = click.argument("TREE", default=sys.stdin, type=click.File())
 
 
 #      smot tips [<filename>]
-@click.command(help="Print the tree tip labels")
+@click.command()
 @dec_tree
 def tips(tree):
+    """
+    Print the tree tip labels. The order of tips matches the order in the tree
+    (top-to-bottom).
+    """
     import smot.algorithm as alg
 
     tree = read_tree(tree)
     tree.tree = alg.setNLeafs(tree.tree)
 
-    def _fun(b, x):
-        if x.isLeaf:
-            b.append(x.label)
-        return b
-
-    for tip in alg.treefold(tree.tree, _fun, []):
+    for tip in alg.tips(tree.tree):
         print(tip)
 
 
@@ -158,6 +165,13 @@ dec_proportion = click.option(
     help="The proportion of tips in a clade to keep",
 )
 
+dec_number = click.option(
+    "-n",
+    "--number",
+    type=click.IntRange(min=0),
+    help="The total number of tips to keep from each group",
+)
+
 dec_newick = click.option(
     "--newick",
     is_flag=True,
@@ -199,10 +213,22 @@ dec_keep_regex = click.option(
     help="Keep all tips that match this pattern, these tips do not count towards downsampling quotas.",
 )
 
-
-@click.command(
-    help="Equal sampling. Descend from root to tip. At each node, determine if each subtree contains a single factor. If a subtree is not monophyletic, recurse into the subtree. If the subtree is monophyletic, then select up to N tips (from the --max-tips argument) from the subtree. The selection of tips is deterministic but dependent on the ordering of leaves. To sample a subtree, an equal number of tips is sampled from each descendent subtree, and so on recursively down to the tips. The resulting downsampled subtree captures the depth of the tree, but is not representative of the tree's breadth. That is, if N=6 and a tree splits into two subtrees, one with 3 tips and one with 300 tips, still 3 tips will be sampled from each branch."
+dec_impute = click.option(
+    "--impute",
+    is_flag=True,
+    default=False,
+    help="Infer the monophyletic factor from context, if possible",
 )
+
+dec_patristic = click.option(
+    "--patristic",
+    is_flag=True,
+    default=False,
+    help="Infer factors by distance on the tree to the nearest label",
+)
+
+
+@click.command()
 @factoring
 @dec_keep
 @click.option(
@@ -223,6 +249,19 @@ def equal(
     newick,
     tree,
 ):
+    """
+    Equal sampling. Descend from root to tip. At each node, determine if each
+    subtree contains a single factor. If a subtree is not monophyletic, recurse
+    into the subtree. If the subtree is monophyletic, then select up to N tips
+    (from the --max-tips argument) from the subtree. The selection of tips is
+    deterministic but dependent on the ordering of leaves. To sample a subtree,
+    an equal number of tips is sampled from each descendent subtree, and so on
+    recursively down to the tips. The resulting downsampled subtree captures
+    the depth of the tree, but is not representative of the tree's breadth.
+    That is, if N=6 and a tree splits into two subtrees, one with 3 tips and
+    one with 300 tips, still 3 tips will be sampled from each branch.
+    """
+
     import smot.algorithm as alg
 
     tree = read_tree(tree)
@@ -241,9 +280,7 @@ def equal(
         print(sf.nexus(tree))
 
 
-@click.command(
-    help="Proportional sampling. Randomly sample p (0 to 1, from --proportion) tips from each monophyletic (relative to factors) subtree. Retain at least N tips in each branch (--min-tips)."
-)
+@click.command()
 @factoring
 @dec_keep
 @dec_keep_regex
@@ -253,6 +290,7 @@ def equal(
 @dec_min_tips
 @dec_proportion
 @dec_scale
+@dec_number
 @dec_seed
 @dec_newick
 @click.option("--zero", is_flag=True, help="Set branches without lengths to 0")
@@ -267,15 +305,22 @@ def prop(
     min_tips,
     proportion,
     scale,
+    number,
     seed,
     newick,
     zero,
     tree,
 ):
+    """
+    Proportional sampling. Randomly sample p (0 to 1, from --proportion) tips
+    from each monophyletic (relative to factors) subtree. Retain at least N
+    tips in each branch (--min-tips).
+    """
+
     import smot.algorithm as alg
 
-    if not proportion and not scale:
-        die("Please add either a --proportion or --scale option")
+    if not (proportion or scale or number):
+        die("Please add either a --proportion or --scale or --number option")
 
     tree = read_tree(tree)
     tree.tree = factorTree(
@@ -291,6 +336,7 @@ def prop(
         keep_regex=keep_regex,
         proportion=proportion,
         scale=scale,
+        number=number,
         minTips=min_tips,
         seed=seed,
     )
@@ -301,9 +347,7 @@ def prop(
         print(sf.nexus(tree))
 
 
-@click.command(
-    help="Paraphyletic sampling. The sampling algorithm starts at the root and descends to the tips. At each node, we store monophyletic subtrees in a list and descend into polyphyletic ones (whose leaves have multiple factors). If we reach a tip or encounter a monophyletic child of a different factor than the stored subtrees, then we stop and sample from all tips in the stored trees and initialize a new list with the new monophyletic child."
-)
+@click.command()
 @factoring
 @dec_keep
 @dec_keep_regex
@@ -313,6 +357,7 @@ def prop(
 @dec_min_tips
 @dec_proportion
 @dec_scale
+@dec_number
 @dec_seed
 @dec_newick
 @click.option("--zero", is_flag=True, help="Set branches without lengths to 0")
@@ -327,15 +372,26 @@ def para(
     min_tips,
     proportion,
     scale,
+    number,
     seed,
     newick,
     zero,
     tree,
 ):
+    """
+    Paraphyletic sampling. The sampling algorithm starts at the root and
+    descends to the tips. At each node, we store monophyletic subtrees in a
+    list and descend into polyphyletic ones (whose leaves have multiple
+    factors). If we reach a tip or encounter a monophyletic child of a
+    different factor than the stored subtrees, then we stop and sample from all
+    tips in the stored trees and initialize a new list with the new
+    monophyletic child.
+    """
+
     import smot.algorithm as alg
 
-    if not proportion and not scale:
-        die("Please add either a --proportion or --scale option")
+    if not (proportion or scale or number):
+        die("Please add either a --proportion or --scale or --number option")
 
     tree = read_tree(tree)
     tree.tree = factorTree(
@@ -351,6 +407,7 @@ def para(
         keep_regex=keep_regex,
         proportion=proportion,
         scale=scale,
+        number=number,
         minTips=min_tips,
         seed=seed,
     )
@@ -361,9 +418,7 @@ def para(
         print(sf.nexus(tree))
 
 
-@click.command(
-    help="Impute, annotate with, and/or tabulate factors. The --impute option will fill in missing factors in monophyletic branches. This is useful, for example, for inferring clades given a few references in a tree. There are three modes: 'table' prints a TAB-delimited table of tip names and factors, 'prepend' adds the factor to the beginning of the tiplabel (delimited with '|'), 'append' adds it to the end."
-)
+@click.command()
 @click.argument(
     "method", type=click.Choice(["table", "prepend", "append"], case_sensitive=False)
 )
@@ -374,18 +429,8 @@ def para(
     default=None,
     help="The name to assign to tips that do not match a factor",
 )
-@click.option(
-    "--impute",
-    is_flag=True,
-    default=False,
-    help="Infer the monophyletic factor from context, if possible",
-)
-@click.option(
-    "--patristic",
-    is_flag=True,
-    default=False,
-    help="Infer factors by distance on the tree to the nearest label",
-)
+@dec_impute
+@dec_patristic
 @dec_newick
 @dec_tree
 def factor(
@@ -399,6 +444,15 @@ def factor(
     newick,
     tree,
 ):
+    """
+    Impute, annotate with, and/or tabulate factors. The --impute option will
+    fill in missing factors in monophyletic branches. This is useful, for
+    example, for inferring clades given a few references in a tree. There are
+    three modes: 'table' prints a TAB-delimited table of tip names and factors,
+    'prepend' adds the factor to the beginning of the tiplabel (delimited with
+    '|'), 'append' adds it to the end.
+    """
+
     import smot.algorithm as alg
 
     tree = read_tree(tree)
@@ -408,13 +462,10 @@ def factor(
         factor_by_field=factor_by_field,
         factor_by_table=factor_by_table,
         default=default,
+        impute=impute,
+        patristic=patristic,
     )
 
-    if patristic:
-        tree.tree = alg.imputePatristicFactors(tree.tree)
-    elif impute:
-        tree.tree = alg.imputeFactors(tree.tree)
-
     # create TAB-delimited, table with columns for the tip labels and the
     # (possibly imputed) factor
     if method.lower() == "table":
@@ -453,12 +504,16 @@ def factor(
 
 
 #      smot tipsed <pattern> <replacement> [<filename>]
-@click.command(help="Search and replace patterns in tip labels")
+@click.command()
 @click.argument("PATTERN", type=str)
 @click.argument("REPLACEMENT", type=str)
 @dec_newick
 @dec_tree
 def tipsed(pattern, replacement, newick, tree):
+    """
+    Search and replace patterns in tip labels.
+    """
+
     import smot.algorithm as alg
     import re
 
@@ -478,7 +533,7 @@ def tipsed(pattern, replacement, newick, tree):
         print(sf.nexus(tree))
 
 
-@click.command(help="Prune a tree to preserve only the tips with that match a pattern")
+@click.command()
 @click.argument("PATTERN", type=str)
 @click.option(
     "-v", "--invert-match", is_flag=True, help="Keep all leafs NOT matching the pattern"
@@ -495,6 +550,10 @@ def tipsed(pattern, replacement, newick, tree):
 @dec_newick
 @dec_tree
 def grep(pattern, tree, invert_match, perl, newick, file):
+    """
+    Prune a tree to preserve only the tips with that match a pattern.
+    """
+
     import smot.algorithm as alg
     import re
 
@@ -528,6 +587,144 @@ def grep(pattern, tree, invert_match, perl, newick, file):
         print(sf.nexus(tree))
 
 
+@click.command(name="filter")
+# conditions used to select groups upon which an action is performed
+@click.option(
+    "--all-match", multiple=True, help="Select if all tips match this expression"
+)
+@click.option(
+    "--some-match",
+    multiple=True,
+    help="Select if one or more tips match this expression",
+)
+@click.option(
+    "--none-match", multiple=True, help="Select if no tips match this expression"
+)
+@click.option(
+    "--larger-than",
+    type=int,
+    help="Select if the number of tips is larger than or equal to this number",
+)
+@click.option(
+    "--smaller-than",
+    type=int,
+    help="Select if the number of tips is smaller than or equal to this number",
+)
+# actions
+@click.option("--remove", is_flag=True, help="Remove group if all conditions are met")
+@click.option(
+    "--color",
+    type=str,
+    help="Color the clade branches this color if all conditions are met",
+)
+@click.option(
+    "--sample",
+    type=float,
+    help="Sample this proportion (0-1) of the group if all conditions are met",
+)
+@click.option(
+    "--replace",
+    nargs=2,
+    help="Replace arg1 with arg2 in all leaf names in group if all conditions are met",
+)
+@factoring
+@dec_patristic
+@dec_seed
+@dec_newick
+@dec_tree
+def filter_cmd(
+    # conditions
+    all_match,
+    some_match,
+    none_match,
+    larger_than,
+    smaller_than,
+    # actions
+    remove,
+    color,
+    sample,
+    replace,
+    # factor methods
+    factor_by_capture,
+    factor_by_field,
+    factor_by_table,
+    # phylogenetic options
+    patristic,
+    seed,
+    # boilerplate
+    newick,
+    tree,
+):
+    """
+    An advanced tool for performaing actions (remove, color, sample, or
+    replace) on monophyletic groups that meet specified conditions (all-match,
+    some-match, etc.
+    """
+    import smot.algorithm as alg
+    import re
+
+    tree = read_tree(tree)
+    tree.tree = factorTree(
+        node=tree.tree,
+        factor_by_capture=factor_by_capture,
+        factor_by_field=factor_by_field,
+        factor_by_table=factor_by_table,
+        patristic=patristic,
+    )
+
+    def condition(node):
+        tips = alg.tips(node)
+        return (
+            (not larger_than or len(tips) > larger_than)
+            and (not smaller_than or len(tips) < smaller_than)
+            and (
+                not all_match
+                or all(
+                    [all([re.search(pat, tip) for tip in tips]) for pat in all_match]
+                )
+            )
+            and (
+                not some_match
+                or all(
+                    [any([re.search(pat, tip) for tip in tips]) for pat in some_match]
+                )
+            )
+            and (
+                not none_match
+                or all(
+                    [
+                        all([not re.search(pat, tip) for tip in tips])
+                        for pat in none_match
+                    ]
+                )
+            )
+        )
+
+    if remove:
+        action = lambda x: None
+    elif color:
+        action = lambda x: alg.colorTree(x, color)
+    elif sample:
+        action = lambda x: sampleProportional(
+            x, proportion=sample, scale=None, minTips=3, keep_regex="", seed=seed
+        )
+    elif replace:
+
+        def _fun(d):
+            d.label = re.sub(replace[0], replace[1], d.label)
+            return d
+
+        action = lambda x: alg.treemap(x, _fun)
+
+    tree.tree = alg.filterMono(tree.tree, condition=condition, action=action)
+    tree.tree = alg.clean(tree.tree)
+
+    if newick:
+        print(sf.newick(tree))
+    else:
+        print(sf.nexus(tree))
+
+
 @click.command()
 @click.option("-p", "--pattern", nargs=2, multiple=True)
 @click.option(
@@ -545,12 +742,7 @@ def leaf(pattern, perl, tree):
 
     tree = read_tree(tree)
 
-    def _fun(b, x):
-        if x.isLeaf:
-            b.append(x.label)
-        return b
-
-    tips = alg.treefold(tree.tree, _fun, [])
+    tips = alg.tips(tree.tree)
 
     for (pat_str, col) in pattern:
         if perl:
@@ -601,7 +793,7 @@ def chooseColorScheme(factors):
     else:
         die("I can't handle more than 11 colors yet")
 
-    colormap = {f : c for (f,c) in  zip(factors, colors)}
+    colormap = {f: c for (f, c) in zip(factors, colors)}
 
     return colormap
 
@@ -627,7 +819,10 @@ def colorBranches(
     if colormap:
         with open(colormap, "r") as f:
             try:
-                _colormap = {f: c for (f, c) in [p.split("\t") for p in f.readlines()]}
+                _colormap = {
+                    f.strip(): c.strip()
+                    for (f, c) in [p.split("\t") for p in f.readlines()]
+                }
             except ValueError:
                 die("Invalid color map: expected TAB-delimited, two-column file")
     else:
@@ -667,6 +862,35 @@ def para_color_cmd(**kwargs):
     colorBranches(is_para=True, **kwargs)
 
 
+@click.command(name="rm")
+@click.option(
+    "--newick",
+    is_flag=True,
+    help="Write output in newick format (metadata will be lost)",
+)
+@dec_tree
+def rm_color(newick, tree):
+    """
+    Remove all color annotations from a tree
+    """
+    import smot.algorithm as alg
+
+    tree = read_tree(tree)
+    tree.colmap = dict()
+
+    def _fun(d):
+        if d.form and "!color" in d.form:
+            del d.form["!color"]
+        return d
+
+    tree.tree = alg.treemap(tree.tree, _fun)
+
+    if newick:
+        print(sf.newick(tree))
+    else:
+        print(sf.nexus(tree))
+
+
 CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
 
@@ -675,11 +899,20 @@ def cli():
     pass
 
 
-@click.group(
-    help="Subsample the tree using various methods. The details of the sampling algorithms differ, but they all start by adding 0 or 1 labels (or factors) to each tip in the tree. These factors are assigned in 1 of 3 ways, described in the --factor-by-capture, --factor-by-field, and --factor-by-table options. Once the factors have been determined, we ascend from tip to root recording the set of all descendent factors in each node. Thus the ancestral node of a monophyletic subtree, where all leaves have the same factor (or no factor), will store a set of exactly one factor. The resulting factored tree is the starting data structure for each of the sampling algorithms.",
-    context_settings=CONTEXT_SETTINGS,
-)
+@click.group(context_settings=CONTEXT_SETTINGS)
 def sample():
+    """
+    Subsample the tree using various methods. The details of the sampling
+    algorithms differ, but they all start by adding 0 or 1 labels (or factors)
+    to each tip in the tree. These factors are assigned in 1 of 3 ways,
+    described in the --factor-by-capture, --factor-by-field, and
+    --factor-by-table options. Once the factors have been determined, we ascend
+    from tip to root recording the set of all descendent factors in each node.
+    Thus the ancestral node of a monophyletic subtree, where all leaves have
+    the same factor (or no factor), will store a set of exactly one factor. The
+    resulting factored tree is the starting data structure for each of the
+    sampling algorithms.
+    """
     pass
 
 
@@ -690,7 +923,11 @@ sample.add_command(para)
 
 @click.group()
 def branch():
-    "Color the branches of a tree. You may provide a color map; if you do not, smot will automatically map factors to colors from a color-blind friendly palette."
+    """
+    Color the branches of a tree. You may provide a color map; if you do not,
+    smot will automatically map factors to colors from a color-blind friendly
+    palette.
+    """
     pass
 
 
@@ -698,22 +935,32 @@ branch.add_command(mono_color_cmd)
 branch.add_command(para_color_cmd)
 
 
-@click.group(
-    help="Color the tips or branches. The coloring options are highly opinionated. Leaf colors are based on patterns inferred from leaf labels. They are generally independent of the leaf context within the tree. There is no direct way to color leafs by clade (and I don't think there should be). Group coloring should be done at the branch color level. Branch coloring is explicitly phylogenetic - you may color branches that monophyletic or paraphyletic for a given factor. There is no simple way to color a particular branch (and why would you want to do that anyway). So, follow my tree coloring dogma and everything will be fine.",
-    context_settings=CONTEXT_SETTINGS,
-)
+@click.group(context_settings=CONTEXT_SETTINGS)
 def color():
+    """
+    Color the tips or branches. The coloring options are highly opinionated.
+    Leaf colors are based on patterns inferred from leaf labels. They are
+    generally independent of the leaf context within the tree. There is no
+    direct way to color leafs by clade (and I don't think there should be).
+    Group coloring should be done at the branch color level. Branch coloring
+    is explicitly phylogenetic - you may color branches that monophyletic or
+    paraphyletic for a given factor. There is no simple way to color a
+    particular branch (and why would you want to do that anyway). So, follow
+    my tree coloring dogma and everything will be fine.
+    """
     pass
 
 
 color.add_command(leaf)
 color.add_command(branch)
+color.add_command(rm_color)
 
 cli.add_command(tips)
 cli.add_command(sample)
 cli.add_command(factor)
 cli.add_command(tipsed)
 cli.add_command(grep)
+cli.add_command(filter_cmd)
 cli.add_command(color)
 
 
